/*****************************************************************
|
|      Xaudio Player for Windows CE
|      File Routines      
|
|      (c) 1996-1998 MpegTV, LLC
|      Author: Gilles Boccon-Gibod (gilles@mpegtv.com)
|
 ****************************************************************/

/*----------------------------------------------------------------------
|       includes
+---------------------------------------------------------------------*/
#ifndef STRICT
#define STRICT
#endif

#include "stdafx.h"

//#include <commdlg.h>
//#include <tchar.h>
//#include <winnls.h>

#include "utils.h"

#include "files.h"
#include "mtvp/ceutils.h"


#if defined(POCKETTV_USE_TGETFILE)
#include "tgetfile.h"
#endif

/* number of items in an array */
#define sizeofarray(t) (sizeof(t)/sizeof((t)[0]))

/*----------------------------------------------------------------------
|       ChooseOpenFile
+---------------------------------------------------------------------*/
int
ChooseOpenFile(HWND parent,
			   TCHAR *buffer,
			   unsigned int buffer_size, 
//               FileChooserType type,
			   TCHAR *folder,
			   const TCHAR *caption,
			   const TCHAR *filter,
			   unsigned int filter_index,
			   const TCHAR *default_extension,
			   bool use_tgetfile)
{
	OPENFILENAME ofn = {0};

	ofn.lStructSize	= sizeof(ofn);
	ofn.hwndOwner   = parent;
    ofn.lpstrFile   = buffer;
    ofn.nMaxFile    = buffer_size;	
	ofn.lpstrInitialDir = folder;
    ofn.lpstrTitle  = caption;
    ofn.Flags       = OFN_EXPLORER | OFN_PATHMUSTEXIST;
    ofn.lpstrFilter = filter;
	ofn.nFilterIndex = filter_index;
    ofn.lpstrDefExt = default_extension;

#if defined(POCKETTV_USE_TGETFILE)
	if (use_tgetfile) {
//		bool have_tgetfile_dll = false;
//		static DWORD count = 0;
		BOOL ret;

#if 0
		HANDLE file = CreateFile(TEXT("\\windows\\tgetfile.dll"),
								GENERIC_READ,
								FILE_SHARE_READ,
								NULL,
								OPEN_EXISTING,
								FILE_ATTRIBUTE_NORMAL,
								NULL);

		if (file != INVALID_HANDLE_VALUE) {
			have_tgetfile_dll = true;
			CloseHandle(file);
		}
#endif // 0
		
		ret = tGetOpenFileName(&ofn);
#if 0
		if (!ret) {
			if (have_tgetfile_dll && count % 5 == 0) {
				count++;
				ret = tGetOpenFileName(&ofn);
			}
		}
		if (have_tgetfile_dll) {
			count++;
		}
#endif // 0
		return ret;
	} else {
		return GetOpenFileName(&ofn);
	}
#else
    return GetOpenFileName(&ofn);
#endif
}

/*----------------------------------------------------------------------
| ChooseSaveFile
+---------------------------------------------------------------------*/
int
ChooseSaveFile(HWND parent,
			   TCHAR *buffer,
			   unsigned int buffer_size, 
			   TCHAR *folder,
			   const TCHAR *caption,
			   const TCHAR *filter,
			   unsigned int filter_index,
			   const TCHAR *default_extension,
			   bool use_tgetfile)
{
	OPENFILENAME ofn = {0};

	ofn.lStructSize	= sizeof(ofn);
	ofn.hwndOwner   = parent;
    ofn.lpstrFile   = buffer;
    ofn.nMaxFile    = buffer_size;	
	ofn.lpstrInitialDir = folder;
    ofn.lpstrTitle  = caption;
    ofn.Flags       = OFN_EXPLORER | OFN_OVERWRITEPROMPT;
    ofn.lpstrFilter = filter;
	ofn.nFilterIndex = filter_index;
    ofn.lpstrDefExt = default_extension;

	bool sip_was_on = (bool)HideSip();
	BOOL ret;

#if defined(POCKETTV_USE_TGETFILE)
	if (use_tgetfile) {
		
		ret = tGetSaveFileName(&ofn);
	} else {
		ret = GetSaveFileName(&ofn);
	}
#else
    ret = GetSaveFileName(&ofn);
#endif
	if (sip_was_on) {
		ShowSip();
	} else {
		HideSip();	// only necessary if use_tgetfile, because tGetSaveFileName does not restore the SIB state
	}
	return ret;
}