/*----------------------------------------------------------------------
|       includes
+---------------------------------------------------------------------*/
#ifndef STRICT
#define STRICT
#endif

#include "stdafx.h"

#include "utils.h"

//#include <commdlg.h>
//#include <tchar.h>
//#include <winnls.h>



// pockettv_malloc and pockettv_calloc are smarter replacements for malloc and calloc.
//
// They try to free some memory by asking other applications to terminate gracefully 
// with SHCloseApps if there is not enough memory available.

#ifdef POCKETTV_HAVE_SHCLOSEAPPS

/*----------------------------------------------------------------------
| pockettv_malloc
+---------------------------------------------------------------------*/
void *pockettv_malloc(unsigned int size)
{
    void *memory;

    if (memory = malloc(size)) {
        return memory;
    }
	SHCloseApps(size);

    return malloc(size);
}

/*----------------------------------------------------------------------
| pockettv_calloc
+---------------------------------------------------------------------*/
void *pockettv_calloc(unsigned int num, unsigned int size)
{
    void *memory;

    if (memory = calloc(num, size)) {
        return memory;
    }
	SHCloseApps(num*size);

    return calloc(num, size);
}

#endif // POCKETTV_HAVE_SHCLOSEAPPS

#ifdef POCKETTV_NO_CALLOC

void *calloc(unsigned int num, unsigned int size)
{
    void *memory;

    if (memory = (void *)malloc(num*size)) {
        memset(memory, 0, num*size);
    }

    return memory;
}
#endif // POCKETTV_NO_CALLOC

#ifndef POCKETTV_NO_SIP

#ifdef WIN32_PLATFORM_BE300

/*----------------------------------------------------------------------
| HideSip
+---------------------------------------------------------------------*/
BOOL
HideSip()
{   
    BOOL fRes = FALSE;
    SIPINFO si = {0};
//    memset( &si, 0, sizeof( si ) );

	// returns TRUE if the SIP was ON, FALSE if it was OFF (or if an error occured)
    si.cbSize = sizeof( si );
    if (SipGetInfo(&si)) {
		if (si.fdwFlags & SIPF_ON) {
			// SIP currently ON
			fRes = TRUE;

			if (!SipShowIM(SIPF_OFF)) {
				return FALSE;
			}
		}
	}
    return fRes;
}

/*----------------------------------------------------------------------
| ShowSip
+---------------------------------------------------------------------*/
BOOL
ShowSip()
{   
    BOOL fRes = TRUE;
    SIPINFO si = {0};
//    memset( &si, 0, sizeof( si ) );

    
	// returns FALSE if the SIP was OFF, TRUE if it was ON (or if an error occured)
	si.cbSize = sizeof( si );

    if (SipGetInfo(&si)) {

		if (!(si.fdwFlags & SIPF_ON)) {
			// SIP currently OFF
			fRes = FALSE;

			if (!SipShowIM(SIPF_ON)) {
				return TRUE;
			}
		}
    }    
    return fRes;
}

#else // not WIN32_PLATFORM_BE300

#ifdef SPI_SETSIPINFO

/*----------------------------------------------------------------------
| HideSip
+---------------------------------------------------------------------*/
BOOL
HideSip()
{   
    BOOL fRes = FALSE;
    SIPINFO si = {0};
//    memset( &si, 0, sizeof( si ) );

	// returns TRUE if the SIP was ON, FALSE if it was OFF (or if an error occured)
    si.cbSize = sizeof( si );
    if( SHSipInfo( SPI_GETSIPINFO, 0, &si, 0 ) ) {
		if (si.fdwFlags & SIPF_ON) {
			// SIP currently ON
			fRes = TRUE;

			si.fdwFlags &= ~SIPF_ON;
			if (!SHSipInfo( SPI_SETSIPINFO, 0, &si, 0 )) {
				return FALSE;
			}
		}
	}
    return fRes;
}

/*----------------------------------------------------------------------
| ShowSip
+---------------------------------------------------------------------*/
BOOL
ShowSip()
{   
    BOOL fRes = TRUE;
    SIPINFO si = {0};
//    memset( &si, 0, sizeof( si ) );

    
	// returns FALSE if the SIP was OFF, TRUE if it was ON (or if an error occured)
	si.cbSize = sizeof( si );
    if( SHSipInfo( SPI_GETSIPINFO, 0, &si, 0 ) ) {

		if (!(si.fdwFlags & SIPF_ON)) {
			// SIP currently OFF
			fRes = FALSE;

			si.fdwFlags |= SIPF_ON;
			if (!SHSipInfo( SPI_SETSIPINFO, 0, &si, 0 )) {
				return TRUE;
			}
		}
    }    
    return fRes;
}

#endif // SPI_SETSIPINFO
#endif // not WIN32_PLATFORM_BE300
#endif // #ifndef POCKETTV_NO_SIP


#ifdef POCKETTV_CAN_SET_NAVBAR_TEXT	// if Pocket PC

/*--------------------------------------------------------------------------+
| SetNavBarText
+--------------------------------------------------------------------------*/
int
SetNavBarText(HWND hWnd, const TCHAR *str)
{
//	return SHSetNavBarText(hWnd, str);
//#if 0
	int		status;

	if (str == NULL) {
		TCHAR buf[30];

		GetWindowText(hWnd, buf, sizeof(buf)/sizeof(buf[0]));
		status = SHSetNavBarText(hWnd, buf);
	} else {
		status = SHSetNavBarText(hWnd, str);
	}
	return status;
//#endif // 0
}


#endif

/*--------------------------------------------------------------------------+
| MyLoadString
+--------------------------------------------------------------------------*/
const TCHAR *
MyLoadString(HINSTANCE instance, int id)
{
	unsigned int n;
	TCHAR *q;
	const TCHAR *p = (const TCHAR *)LoadString(instance, id, 0, 0);
	if (p == NULL) {
		return NULL;
	}
	n = *((WORD *)p - 1);
	q = (TCHAR *)malloc((n + 1) * sizeof(TCHAR));
	if (q == NULL) {
		return NULL;
	}
	LoadString(instance, id, q, n + 1);
//	q[n] = 0;
	return q;
}

/*--------------------------------------------------------------------------+
| MyFreeString
+--------------------------------------------------------------------------*/
void
MyFreeString(const TCHAR *str)
{
	if (str != NULL) {
		free((void *)str);
	}
}

#if !defined(UNDER_CE) || UNDER_CE < 212 || !defined(WIN32_PLATFORM_PSPC)

/*----------------------------------------------------------------------
| center_dialog
+---------------------------------------------------------------------*/
void
center_dialog(HWND hDlg) {
	RECT rt, rt1;
	int DlgWidth, DlgHeight;	// dialog width and height in pixel units
	int NewPosX, NewPosY;

	// trying to center the dialog window

	// bug: this does not take the SIP into account:
	// if the SIP is visible, the window is NOT properly centered...

	if (GetWindowRect(hDlg, &rt1)) {
		GetClientRect(GetParent(hDlg), &rt);
		DlgWidth	= rt1.right - rt1.left;
		DlgHeight	= rt1.bottom - rt1.top ;
		NewPosX		= (rt.right - rt.left - DlgWidth)/2;
		NewPosY		= (rt.bottom - rt.top - DlgHeight)/2;
		
		// if the About box is larger than the physical screen 
		if (NewPosX < 0) NewPosX = 0;
		if (NewPosY < 0) NewPosY = 0;
		SetWindowPos(hDlg, 0, NewPosX, NewPosY,
			0, 0, SWP_NOZORDER | SWP_NOSIZE);
	}
}

/*----------------------------------------------------------------------
| fullscreen_dialog
+---------------------------------------------------------------------*/
void
fullscreen_dialog(HWND hDlg) {
	RECT r;

	SystemParametersInfo(SPI_GETWORKAREA, 0, &r, 0);

	SetWindowPos(hDlg, 0, r.left, r.top,
			r.right - r.left,
			r.bottom - r.top,
			SWP_NOZORDER);
}

/*----------------------------------------------------------------------
| adjust_dialog
+---------------------------------------------------------------------*/
void
adjust_dialog(HWND hDlg) {

#if defined(WIN32_PLATFORM_PSPC) || defined(WIN32_PLATFORM_BE300)
	fullscreen_dialog(hDlg);
#else
	center_dialog(hDlg);
#endif

}
#endif

/*--------------------------------------------------------------------------+
| SetWindowTitle
+--------------------------------------------------------------------------*/
int
SetWindowTitle(HWND hWnd, const TCHAR *str)
{
//	return SHSetNavBarText(hWnd, str);
//#if 0
//	int		status;
	BOOL	success;

	if (str == NULL) {
//		TCHAR buf[30];

		success = SetWindowText(hWnd, TEXT("PocketTV"));

//		status = SHSetNavBarText(hWnd, buf);
	} else {
		success = SetWindowText(hWnd, str);
//		status = SHSetNavBarText(hWnd, str);
	}
	if (success) return 0; else return -1;
//	return status;
//#endif // 0
}
